import React from "react";
import { Helmet } from "react-helmet-async";
import { Route, Routes, useLocation } from "react-router-dom";
import PageNotFound from "../common/lib/PageNotFound";
import { BrainAtlasType, MultiLang } from "../config";
import Functions from "../functions";
import styles from "./Database.module.css";
import DatabaseAdvancedSearch from "./DatabaseAdvancedSearch";
import DatabaseDetailItem from "./DatabaseDetailItem";
import DatabaseSearchByAdvancedKeyword from "./DatabaseSearchByAdvancedKeyword";
import DatabaseSearchByIndexId from "./DatabaseSearchByIndexId";
import DatabaseSearchByItemType from "./DatabaseSearchByItemType";
import DatabaseSearchByKeyword from "./DatabaseSearchByKeyword";
import DatabaseTop from "./DatabaseTop";
import { INDEX_ID_PUBLIC } from "./lib/IndexUtil";

interface Props {
  lang: MultiLang;
  type: BrainAtlasType;
}

const ItemDetail: React.FC<Props> = (props: Props) => {
  const { lang, type } = props;
  const location = useLocation();
  const params = new URLSearchParams(location.search);
  const itemId_ = params.get("item_id") || "";
  const itemId = /^\d+$/.test(itemId_) ? parseInt(itemId_, 10) : 0;
  const doi = params.get("id") || "";
  return <DatabaseDetailItem lang={lang} id={itemId} doi={doi} type={type} />;
};

const ItemList: React.FC<Props> = (props: Props) => {
  const { lang, type } = props;
  const location = useLocation();
  const params = new URLSearchParams(location.search);
  const id = params.get("index_id") || "";
  const indexId = /^\d+$/.test(id) ? parseInt(id, 10) : INDEX_ID_PUBLIC;
  return <DatabaseSearchByIndexId lang={lang} type={type} indexId={indexId} />;
};

const ItemSelect: React.FC<Props> = (props: Props) => {
  const { lang, type } = props;
  const location = useLocation();
  const params = new URLSearchParams(location.search);
  const op = params.get("op") || "";
  switch (op) {
    case "itemtypesearch": {
      const searchItemtype = params.get("search_itemtype") || "";
      const match = searchItemtype.match(/^xnp([a-z]+)$/);
      const itemType = match !== null ? match[1] : "";
      return <DatabaseSearchByItemType lang={lang} itemType={itemType} subItemType="" type={type} />;
    }
    case "itemsubtypesearch": {
      const searchItemtype = params.get("search_itemtype") || "";
      const match = searchItemtype.match(/^xnp([a-z]+)$/);
      const itemType = match !== null ? match[1] : "";
      const subItemtype = params.get("search_subitemtype") || "";
      return <DatabaseSearchByItemType lang={lang} itemType={itemType} subItemType={subItemtype} type={type} />;
    }
    case "quicksearch": {
      return <DatabaseSearchByKeyword lang={lang} type={type} />;
    }
    case "advanced": {
      return <DatabaseSearchByAdvancedKeyword lang={lang} type={type} />;
    }
  }
  return <PageNotFound lang={lang} />;
};

const Database: React.FC<Props> = (props: Props) => {
  const { lang, type } = props;
  return (
    <div className={styles.database}>
      <Helmet>
        <title>
          {Functions.mlang("[en]Database[/en][ja]データベース[/ja]", lang)} - {Functions.siteTitle(lang)}
        </title>
      </Helmet>
      <Routes>
        <Route index element={<DatabaseTop lang={lang} type={type} />} />
        <Route path="index.php" element={<DatabaseTop lang={lang} type={type} />} />
        <Route path="detail.php" element={<ItemDetail lang={lang} type={type} />} />
        <Route path="listitem.php" element={<ItemList lang={lang} type={type} />} />
        <Route path="itemselect.php" element={<ItemSelect lang={lang} type={type} />} />
        <Route path="advanced_search.php" element={<DatabaseAdvancedSearch lang={lang} type={type} />} />
        <Route path="*" element={<PageNotFound lang={lang} />} />
      </Routes>
    </div>
  );
};

export default Database;
