import loki from "lokijs";
import { BrainAtlasType } from "../../config.js";
import treesJson from "../assets/trees.json";

export interface Index {
  id: number;
  title: string;
  numOfItems: number;
  parentId: number;
  weight: number;
}

export const INDEX_ID_ROOT = 1;
export const INDEX_ID_PUBLIC = 3;

interface IndexData {
  id: number;
  title: string;
  num_of_items: number;
  children: IndexData[];
}
type IndexesData = IndexData[];

interface TreeJson {
  type: BrainAtlasType;
  tree: IndexesData;
}
type TreesJson = TreeJson[];

class IndexUtil {
  private database: loki;
  private indexes: Map<BrainAtlasType, Collection<Index>>;

  constructor(trees: TreesJson) {
    this.database = new loki("database");
    this.indexes = new Map<BrainAtlasType, Collection<Index>>();
    trees.forEach((tree) => {
      const collection = this.database.addCollection(tree.type + "_indexes");
      this.load(tree.tree, collection);
      this.indexes.set(tree.type, collection);
    });
  }

  load(json: IndexesData, collection: Collection<Index>): void {
    const store = (indexesData: IndexesData, parentId: number) => {
      indexesData.forEach((indexData: IndexData, idx) => {
        const entry: Index = {
          id: indexData.id,
          title: indexData.title,
          numOfItems: indexData.num_of_items,
          parentId: parentId,
          weight: idx,
        };
        collection.insert(entry);
        store(indexData.children, indexData.id);
      });
    };
    store(json, INDEX_ID_ROOT);
  }

  getUrl(type: BrainAtlasType, id: number): string {
    return "/" + type + "/modules/xoonips/listitem.php?index_id=" + String(id);
  }

  get(type: BrainAtlasType, indexId: number): Index | null {
    const filter = {
      id: indexId,
    };
    const collection = this.indexes.get(type);
    if (typeof collection === "undefined") {
      return null;
    }
    const res = collection.findOne(filter);
    return res;
  }

  getChildren(type: BrainAtlasType, indexId: number): Index[] {
    const filter = {
      parentId: indexId,
    };
    const collection = this.indexes.get(type);
    if (typeof collection === "undefined") {
      return [];
    }
    const res = collection.chain().find(filter).simplesort("weight").data();
    return res;
  }

  countChildren(type: BrainAtlasType, indexId: number): number {
    const filter = {
      parentId: indexId,
    };
    const collection = this.indexes.get(type);
    if (typeof collection === "undefined") {
      return 0;
    }
    const res = collection.count(filter);
    return res;
  }

  getParents(type: BrainAtlasType, parentId: number): Index[] {
    let parents: Index[] = [];
    const loop = (parentId: number) => {
      if (parentId !== INDEX_ID_ROOT) {
        const parent = this.get(type, parentId);
        if (parent !== null) {
          loop(parent.parentId);
          parents.push(parent);
        }
      }
    };
    loop(parentId);
    return parents;
  }
}

export default new IndexUtil(treesJson as TreesJson);
